#include "fsm.h"

FSM::FSM()
{
	numStates=0;
	currentState=0;
	states = NULL;

	maxTransitions=0;
	numTransitions=0;
	transitions=NULL;
	running = false;
}

FSM::FSM(uint8_t nstates, uint8_t ntransitions)
{
		init(nstates, ntransitions);
}

void 
FSM::init(uint8_t nstates, uint8_t ntransitions)
{
	uint8_t i;

	numStates = nstates;
	states = new FSM_STATE [ numStates ];
	currentState=0;
	for (i=0; i<numStates; i++) 
	{
		states[i].OnEnter=NULL;
		states[i].OnState=NULL;
		states[i].OnLeave=NULL;
	}

	maxTransitions = ntransitions;
	transitions = new FSM_TRANSITION [ maxTransitions ];
	numTransitions = 0;
	for (i=0; i<maxTransitions; i++)
	{
		transitions[i].action=NULL;
		transitions[i].condition=NULL;
	}
}

int 
FSM::addTransition(uint8_t _origin, uint8_t _destiny, FSM_CONDITION condition, FSM_ACTION action=NULL)
{
		if ( numTransitions >= maxTransitions ) return numTransitions;
		transitions[numTransitions].origin = _origin;
		transitions[numTransitions].destiny = _destiny;
		transitions[numTransitions].condition = condition;
		transitions[numTransitions].action = action;
		return numTransitions++;
}

void
FSM::setState(uint8_t state)
{
		if ( states[currentState].OnLeave!=NULL ) states[currentState].OnLeave();
		currentState = state;
		if ( states[currentState].OnEnter!=NULL ) states[currentState].OnEnter();

}

bool 
FSM::update()
{
	  if ( running == false ) return false;
		uint8_t i;
		for (i=0; i<numTransitions; i++)
		{
				if ( transitions[i].origin == currentState && transitions[i].condition!=NULL )
			  {
						if ( transitions[i].condition() )
						{
								if ( transitions[i].action!=NULL ) transitions[i].action();
								setState(transitions[i].destiny);
								return true;
						}
				}
		}
		if ( states[currentState].OnState!=NULL ) states[currentState].OnState();
		return true;
}

void FSM::start()
{
	currentState=0;
	running = true;
	if ( states[currentState].OnEnter!=NULL ) states[currentState].OnEnter();
}

void FSM::stop() {
	running = false;
	currentState=0;
}

