#ifndef _FSM_H_
#define _FSM_H_

#include "Arduino.h"

typedef bool (*FSM_CONDITION)();
typedef void (*FSM_ACTION)();

typedef struct
{
  FSM_ACTION OnEnter;
	FSM_ACTION OnState;
	FSM_ACTION OnLeave;
} FSM_STATE;

typedef struct
{
		uint8_t origin;
		uint8_t destiny;
		FSM_ACTION action;
		FSM_CONDITION condition;
} FSM_TRANSITION;

class FSM
{
		private:
			  // ESTADOS.
				FSM_STATE *states;
			  uint8_t    numStates;
				uint8_t		 currentState;
        bool       running;
				// TRANSICIONES.
				FSM_TRANSITION *transitions;
				uint8_t    maxTransitions;
				uint8_t    numTransitions;

		public:
				FSM();
				FSM(uint8_t nstates, uint8_t ntransition);
				void init(uint8_t nstates, uint8_t ntransitions);
				void setOnEnter(uint8_t state, FSM_ACTION action) { states[state].OnEnter = action; }
				void setOnLeave(uint8_t state, FSM_ACTION action) { states[state].OnLeave = action; }
				void setOnState(uint8_t state, FSM_ACTION action) { states[state].OnState = action; }
				int  addTransition(uint8_t _origin, uint8_t _destiny, FSM_CONDITION condition, FSM_ACTION action);
				void setState(uint8_t state);
				void start();
				void stop();
				bool isRunning() { return running; }
        uint8_t getState() { return currentState; }
				bool update();
};

#endif